/**
 * Svelte action to initialize semantic modal dialogue component.
 * @module components/use-modal
 */
import { get, writable } from "svelte/store";
import { jQueryElem, uid, SVELTE_DATA_STORE } from "../data/common";
export const modalDefaults = {};
/**
 * Initializes Semantic UI Modal componenet. Takes settings object as argument.
 *
 * https://semantic-ui.com/modules/modal.html
 *
 * Example:
```
    <div class="ui basic modal"
        use:modal={{ onApprove: okFn, closable: false }}>
        <Data bind:active={show} />
        ...
    </div>
```
or
```
    <div  class="ui basic modal"
        use:modal={{ onApprove: okFn, closable: false }}>
        use:controller={ctrl} />
        ...
    </div>
    <script>
        ctrl.modal("show");
    </script>
```
*/
export function modal(node, settings) {
    const elem = jQueryElem(node);
    if (!elem.modal) {
        throw new Error("Semantic UI is not initialized");
    }
    /*
            dP
            88
 .d8888b. d8888P .d8888b. 88d888b. .d8888b.
 Y8ooooo.   88   88'  `88 88'  `88 88ooood8
       88   88   88.  .88 88       88.  ...
 `88888P'   dP   `88888P' dP       `88888P'

    */
    // create store to push data back to the binder
    const holder = {
        uid: uid(),
        mode: "modal",
        store: writable(),
        /** Push value into the modal. */
        doUpdate(value) {
            if (value) {
                if (!elem.modal("is active")) {
                    elem.modal("show");
                }
            }
            else {
                if (elem.modal("is active")) {
                    elem.modal("hide");
                }
            }
        },
        /** Return updated value from the modal. */
        onChange(newValue) {
            console.debug(`  onChange(${this.uid}) = ${newValue}`);
            if (get(this.store) !== newValue) {
                console.debug(`  store(${this.uid}) <- modal = ${newValue}`);
                this.store.set(newValue);
            }
        },
    };
    function onModalShow() {
        if (modalDefaults.onShow) {
            modalDefaults.onShow();
        }
        if (settings && settings.onShow) {
            settings.onShow();
        }
        holder.onChange(true);
    }
    function onModalHidden() {
        if (modalDefaults.onHidden) {
            modalDefaults.onHidden();
        }
        if (settings && settings.onHidden) {
            settings.onHidden();
        }
        holder.onChange(false);
    }
    /*
 oo          oo   dP
                  88
 dP 88d888b. dP d8888P
 88 88'  `88 88   88
 88 88    88 88   88
 dP dP    dP dP   dP

    */
    // Initialize Semantic component
    elem.modal({
        ...modalDefaults,
        ...settings,
        onShow: onModalShow,
        onHidden: onModalHidden,
    });
    // Attach store holder to jQuery element
    console.debug(`  store(${holder.uid}) - ${holder.mode} created`);
    elem.data(SVELTE_DATA_STORE, holder);
    // IMPORTANT:
    // elem must be removed throght jQuery as modal() function has moved it in DOM
    // and Svelte no longer knows where it is
    return {
        destroy() {
            console.debug("  action - destroy");
            elem.remove();
        },
    };
}
